@@ -1,4 +1,4 @@
-/* KDevelop xUnit plugin
+/* KDevelop coverage plugin
  *    Copyright 2008 Manuel Breugelmans <mbr.nxi@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -26,15 +26,9 @@
 #include "lcovinfoparser.h"
 #include "lcovjob.h"
 #include "covoutputdelegate.h"
+#include "ui_reportwidget.h"
 
 #include <QTimer>
-#include <QLabel>
-#include <QLineEdit>
-#include <QGridLayout>
-#include <QScrollArea>
-#include <QHBoxLayout>
-#include <QPushButton>
-#include <KIconLoader>
 #include <KLocale>
 #include <KIcon>
 #include <KProcess>
@@ -62,18 +56,10 @@ using Veritas::DrillDownView;
 using Veritas::LcovInfoParser;
 using Veritas::LcovJob;
 using Veritas::ReportWidget;
+using Veritas::ReportDirData;
 using Veritas::ReportModel;
 using Veritas::ReportViewFactory;
 
-namespace {
-class FilterBox : public QLineEdit
-{
-public:
-    FilterBox(QWidget* parent) : QLineEdit(parent) {}
-    QSize sizeHint() const { return QSize(500, 20); }
-};
-}
-
 //////////////////////////////// ReportWidget ///////////////////////////////
 
 void ReportWidget::resizeEvent(QResizeEvent* event)
@@ -98,74 +84,23 @@ bool ReportWidget::eventFilter(QObject* obj, QEvent* event)
 
 DrillDownView* ReportWidget::table() const
 {
-    return m_table;
+    return m_ui->table;
 }
 
 void ReportWidget::init()
 {
-    QVBoxLayout *l = new QVBoxLayout();
-    l->setMargin(5);
-    l->setSpacing(5);
+    m_ui = new Ui::ReportWidget();
+    m_ui->setupUi(this);
 
-    QFrame* sel = new QFrame(this); // uppermost frame. contains a breadcrumb selector widget + 'go' button
-    sel->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
-    l->addWidget(sel);
-    QHBoxLayout* selLay = new QHBoxLayout;
-    sel->setLayout(selLay);
-    QLabel* buildDir = new QLabel(i18n("Build Path: "), this);
-    selLay->addWidget(buildDir);
+    //KUrlNavigator can't be set using a placeholder in Qt Designer as there is 
+    //no KUrlNavigator(QWidget*) constructor
+    int startButtonIndex = m_ui->buildPathFrameLayout->indexOf(m_ui->startButton);
     m_targetDirectory = new KUrlNavigator(0, KUrl(QDir::homePath()), this);
-    selLay->addWidget(m_targetDirectory);
-    m_startButton = new QPushButton(KIcon("arrow-right"), QString(), this);
-    selLay->addWidget(m_startButton);
-    connect(m_startButton, SIGNAL(clicked(bool)), SLOT(startLcovJob()));
+    m_ui->buildPathFrameLayout->insertWidget(startButtonIndex, m_targetDirectory);
+    
+    m_ui->startButton->setIcon(KIcon("arrow-right"));
+    connect(m_ui->startButton, SIGNAL(clicked(bool)), SLOT(startLcovJob()));
     
-    QFrame* f = new QFrame(this); // frame that shows filter box + Line Coverage; Instrumented lines & sloc metrics
-    f->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
-    l->addWidget(f);
-
-    QGridLayout* gl = new QGridLayout;
-    f->setLayout(gl);
-
-    QLabel* covTextLabel = new QLabel(i18n("Line Coverage:"), this);
-    QLabel* slocTextLabel = new QLabel(i18n("Total SLOC:"), this);
-    QLabel* instrumentedLabel = new QLabel(i18n("Instrumented Lines:"), this);
-
-    m_coverage = new QLabel("-", this);
-    m_sloc = new QLabel("-", this);
-    m_instrumented = new QLabel("-", this);
-    QSpacerItem* s1 = new QSpacerItem(300,5, QSizePolicy::Expanding, QSizePolicy::Minimum);
-    QSpacerItem* s2 = new QSpacerItem(300,5, QSizePolicy::Expanding, QSizePolicy::Minimum);
-
-    // gl | <-s1-> lineedit     <-s2->|
-    //    |        label1: val1       |
-    //    |        label2: val2       |
-    //    |        label3: val3       |
-
-
-    gl->addItem(s1, 0, 0, 4, 1); // rowspan 4    
-    m_filterBox = new FilterBox(this);
-    m_filterBox->resize(250, m_filterBox->height());
-    gl->addWidget(m_filterBox, 0, 1, 1, 2);
-
-    const int FIRST_COL=1;
-    gl->addWidget(covTextLabel, 1, FIRST_COL); // row, col, rowspan, colspan
-    gl->addWidget(m_coverage, 1, FIRST_COL+1);
-    gl->addWidget(instrumentedLabel, 2, FIRST_COL);
-    gl->addWidget(m_instrumented, 2, FIRST_COL+1);
-    gl->addWidget(slocTextLabel, 3, FIRST_COL);
-    gl->addWidget(m_sloc, 3, FIRST_COL+1);
-    gl->addItem(s2, 0, 3, 4, 1);
-
-    l->addWidget(table());
-    setLayout(l);
-
-    connect(table()->selectionModel(),
-            SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
-            SLOT(dispatchSelectionSignal(QItemSelection,QItemSelection)));
-
-    connect(table(),  SIGNAL(clicked(QModelIndex)),
-            SLOT(dispatchClickedSignal(QModelIndex)));
     connect(table(), SIGNAL(doubleClicked(QModelIndex)),
             SLOT(dispatchDoubleClickedSignal(QModelIndex)));
 
@@ -187,19 +122,18 @@ void ReportWidget::updateTableView()
 }
 
 ReportWidget::ReportWidget(QWidget* parent) : 
-    QWidget(parent), 
-    m_table(new  DrillDownView(this)),
+    QWidget(parent),
     m_state(ReportWidget::DirView),
     m_manager(0),
     m_proxy(0),
     m_model(0),
-    m_timer(new QTimer),
-    m_delegate(new CovOutputDelegate)
+    m_timer(new QTimer(this)),
+    m_targetDirectory(0),
+    m_delegate(new CovOutputDelegate(this)),
+    m_ui(0)
 {
     setObjectName("Coverage Report");
     setWindowIcon(SmallIcon("system-file-manager"));
-    setWindowTitle(i18n("Coverage Report"));
-    setWhatsThis(i18n("GCOV code coverage statistics."));
 }
 
 #define ILLEGAL_STATE \
@@ -218,18 +152,6 @@ void ReportWidget::dispatchSelectionSignal(const QItemSelection& selected, const
     }}
 }
 
-void ReportWidget::dispatchClickedSignal(const QModelIndex& index)
-{
-    switch(m_state) {
-    case DirView: {
-        setCoverageStatistics(index); break;
-    } case FileView: {
-        /*jumpToSource(index); */break;
-    } default: {
-        ILLEGAL_STATE;
-    }}
-}
-
 void ReportWidget::dispatchDoubleClickedSignal(const QModelIndex& index)
 {
     switch(m_state) {
@@ -271,9 +193,6 @@ void ReportWidget::reset_()
 {
     m_proxy->invalidate();
     filterBox()->clear();
-    table()->selectionModel()->clearSelection();
-    table()->selectionModel()->clear();
-    table()->selectionModel()->reset();
     table()->viewport()->update();
     m_timerTicks = 0;
     m_timer->start(); // TODO figure out the correct signal instead of this kludge
@@ -281,33 +200,43 @@ void ReportWidget::reset_()
 
 void ReportWidget::setDirViewState()
 {
-    kDebug() << "switching to dir view state";
     m_state = ReportWidget::DirView;
     m_proxy->setDirViewState();
     reset_();
     table()->resizeDirStateColumns();
     filterBox()->setReadOnly(false);
     filterBox()->setText(m_oldDirFilter);
+
+    //When the view slides to the left, the previously selected directories are
+    //selected again. However, until the slide is completed we are in File
+    //state, so the statistics aren't updated.
+    m_ui->sloc->setText("0");
+    m_ui->nrofCoveredLines->setText("0");
+    setCoverageStatistics(table()->selectionModel()->selection(), QItemSelection());
 }
 
 QLineEdit* ReportWidget::filterBox() const
 {
-    return m_filterBox;
+    return m_ui->filterBox;
 }
 
 void ReportWidget::setFileViewState()
 {
-    kDebug() << "switching to file view state";
     m_state = ReportWidget::FileView;
     m_proxy->setFileViewState();
     m_oldDirFilter = filterBox()->text();
     reset_();
     table()->resizeFileStateColumns();
     filterBox()->setReadOnly(true);
+
+    //Set the statistics for the current directory
+    setCoverageStatistics(table()->rootIndex());
 }
 
 ReportWidget::~ReportWidget()
-{}
+{
+    delete m_ui;
+}
 
 QStandardItem* ReportWidget::getItemFromProxyIndex(const QModelIndex& index) const
 {
@@ -321,37 +250,72 @@ QStandardItem* ReportWidget::getItemFromProxyIndex(const QModelIndex& index) con
     return item;
 }
 
-void ReportWidget::setCoverageStatistics(const QModelIndex& index)
+const ReportDirData* ReportWidget::getReportDirDataFromProxyIndex(const QModelIndex& index) const
 {
     QStandardItem* item = getItemFromProxyIndex(index);
-    if (!item || item->type() != ReportModel::Dir) return;
-    ReportDirItem* dir = static_cast<ReportDirItem*>(item);
-    Q_ASSERT(dir);
+    if (!item || item->type() != ReportModel::Dir) return 0;
 
-    m_coverage->setText(QString::number(dir->coverage(), 'f', 1) + " %");
-    m_coverage->update();
-    m_sloc->setText(QString::number(dir->sloc()));
-    m_sloc->update();
-    m_instrumented->setText(QString::number(dir->instrumented()));
-    m_instrumented->update();
+    return &static_cast<ReportDirItem*>(item)->reportDirData();
 }
 
+void ReportWidget::setCoverageStatistics(const ReportDirData& data)
+{
+    //QLocale used as QString::number does not honor the user's locale setting
+    m_ui->coverageRatio->setText(QLocale().toString(data.coverageRatio(), 'f', 1) + " %");
+    m_ui->coverageRatio->update();
+    m_ui->sloc->setText(QString::number(data.sloc()));
+    m_ui->sloc->update();
+    m_ui->nrofCoveredLines->setText(QString::number(data.nrofCoveredLines()));
+    m_ui->nrofCoveredLines->update();
+}
+
+void ReportWidget::setCoverageStatistics(const QModelIndex& index)
+{
+    const ReportDirData* data = getReportDirDataFromProxyIndex(index);
+    if (!data) {
+        return;
+    }
+
+    setCoverageStatistics(*data);
+}
 
 void ReportWidget::setCoverageStatistics(const QItemSelection& selected, const QItemSelection& deselected)
 {
-    Q_UNUSED(deselected);
-    QModelIndexList indexes = selected.indexes();
-    if (indexes.count() < 1) {
-        kDebug() << "Nothing selected";
-        return; // NO-OP if nothing selected
+    ReportDirData fullData;
+    if (m_ui->sloc->text() != "-" && m_ui->nrofCoveredLines->text() != "-") {
+        fullData.setSloc(m_ui->sloc->text().toInt());
+        fullData.setNrofCoveredLines(m_ui->nrofCoveredLines->text().toInt());
+    }
+
+    bool statisticsChanged = false;
+
+    foreach (const QModelIndex& index, selected.indexes()) {
+        const ReportDirData* data = getReportDirDataFromProxyIndex(index);
+        if (data) {
+            statisticsChanged = true;
+            fullData.setSloc(fullData.sloc() + data->sloc());
+            fullData.setNrofCoveredLines(fullData.nrofCoveredLines() + data->nrofCoveredLines());
+        }
+    }
+
+    foreach (const QModelIndex& index, deselected.indexes()) {
+        const ReportDirData* data = getReportDirDataFromProxyIndex(index);
+        if (data) {
+            statisticsChanged = true;
+            fullData.setSloc(fullData.sloc() - data->sloc());
+            fullData.setNrofCoveredLines(fullData.nrofCoveredLines() - data->nrofCoveredLines());
+        }
+    }
+
+    if (statisticsChanged) {
+        setCoverageStatistics(fullData);
     }
-    setCoverageStatistics(indexes.first());
 }
 
 void ReportWidget::startLcovJob()
 {
-    Q_ASSERT(m_delegate); Q_ASSERT(m_targetDirectory); Q_ASSERT(m_startButton->isEnabled());
-    m_startButton->setEnabled(false);
+    Q_ASSERT(m_delegate); Q_ASSERT(m_targetDirectory); Q_ASSERT(m_ui->startButton->isEnabled());
+    m_ui->startButton->setEnabled(false);
     m_state = DirView;
     
     if (m_model) delete m_model;
@@ -365,13 +329,20 @@ void ReportWidget::startLcovJob()
     table()->setModel(m_proxy);
     m_proxy->setSourceModel(m_model);
 
+    //Since Qt 4.1 (or maybe 4.0, but not documented), setting the model
+    //replaces the previous selection model, so it must be connected again
+    //http://doc.trolltech.com/4.1/qabstractitemview.html#setModel
+    connect(table()->selectionModel(),
+            SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
+            SLOT(dispatchSelectionSignal(QItemSelection,QItemSelection)));
+
     LcovJob* job = new LcovJob(m_targetDirectory->url(), m_delegate);
     LcovInfoParser* parser = new LcovInfoParser(job);
 
     connect(parser, SIGNAL(parsedCoverageData(CoveredFile*)), m_model, SLOT(addCoverageData(CoveredFile*)));
     connect(parser, SIGNAL(parsedCoverageData(CoveredFile*)), m_manager, SLOT(addCoverageData(CoveredFile*)));
     connect(m_model, SIGNAL(dataChanged(QModelIndex,QModelIndex)), SLOT(updateColumns()));
-    connect(m_filterBox, SIGNAL(textChanged(QString)), m_proxy, SLOT(setFilterWildcard(QString)));
+    connect(filterBox(), SIGNAL(textChanged(QString)), m_proxy, SLOT(setFilterWildcard(QString)));
 
     job->setDelegate(m_delegate);
     job->setProcess(new KProcess);
@@ -392,8 +363,8 @@ void ReportWidget::updateColumns()
 
 void ReportWidget::jobFinished()
 {
-    Q_ASSERT(!m_startButton->isEnabled());
-    m_startButton->setEnabled(true);
+    Q_ASSERT(!m_ui->startButton->isEnabled());
+    m_ui->startButton->setEnabled(true);
 }
 
 #include "reportwidget.moc"
