@@ -1,4 +1,4 @@
-/* KDevelop xUnit plugin
+/* KDevelop coverage plugin
  *
  * Copyright 2008 Manuel Breugelmans <mbr.nxi@gmail.com>
  *
@@ -74,7 +74,7 @@ void AnnotationManagerTest::initTestCase()
 
 void AnnotationManagerTest::init()
 {
-    m_manager = new AnnotationManager;
+    m_annotationManager = new AnnotationManager;
     m_someUrl = KUrl("foo.cpp");
     m_widget = new QWidget;
     m_lineNr = 10;
@@ -83,7 +83,14 @@ void AnnotationManagerTest::init()
 
 void AnnotationManagerTest::cleanup()
 {
-    //if (m_manager) delete m_manager;
+    if (m_annotationManager) delete m_annotationManager;
+    if (m_widget) delete m_widget;
+    qDeleteAll(m_garbageFiles);
+    m_garbageFiles.clear();
+    qDeleteAll(m_garbageDocs);
+    m_garbageDocs.clear();
+    qDeleteAll(m_garbage);
+    m_garbage.clear();
 }
 
 ///////////////// commands ///////////////////////////////////////////////////
@@ -91,19 +98,19 @@ void AnnotationManagerTest::cleanup()
 void AnnotationManagerTest::instantiate()
 {
     QMap<KUrl, CoveredFile*> files;
-    m_manager->setCoveredFiles(files);
+    m_annotationManager->setCoveredFiles(files);
     KDevDocument* doc = createKDevDocument(m_someUrl); 
-    m_manager->watch(doc);
-    m_manager->stopWatching(doc);
+    m_annotationManager->watch(doc);
+    m_annotationManager->stopWatching(doc);
 }
 
 void AnnotationManagerTest::singleLine()
 {
     // setup
     CoveredFile* f = createCoveredFile(m_lineNr, m_nrofCalls, m_someUrl);
-    m_manager->addCoverageData(f);
+    m_annotationManager->addCoverageData(f);
     KDevDocument* doc = createKDevDocument(f->url());
-    m_manager->watch(doc);
+    m_annotationManager->watch(doc);
 
     View* v = triggerAnnotationsOnView(doc);                // exercise
     assertAnnoCallCountEquals(m_lineNr, m_nrofCalls, v);    // verify
@@ -115,9 +122,9 @@ void AnnotationManagerTest::multipleLines()
     CoveredFile* f = createCoveredFile(m_lineNr, m_nrofCalls, m_someUrl);
     int line2 = 20, callCount2 = 10;
     f->setCallCount(line2, callCount2);
-    m_manager->addCoverageData(f);
+    m_annotationManager->addCoverageData(f);
     KDevDocument* doc = createKDevDocument(f->url());
-    m_manager->watch(doc);
+    m_annotationManager->watch(doc);
 
     View* v = triggerAnnotationsOnView(doc);                 // exercise
     assertAnnoCallCountEquals(m_lineNr, m_nrofCalls, v);     // verify
@@ -128,15 +135,16 @@ void AnnotationManagerTest::multipleViews()
 {
     // setup
     CoveredFile* f = createCoveredFile(m_lineNr, m_nrofCalls, m_someUrl);
-    m_manager->addCoverageData(f);
+    m_annotationManager->addCoverageData(f);
     KDevDocument* doc = createKDevDocument(f->url());
-    m_manager->watch(doc);
+    m_annotationManager->watch(doc);
 
     // exercise
     View* v1 = triggerAnnotationsOnView(doc);
     // inject another view in the stub
     TextDocument* tdoc = (TextDocument*)doc->textDocument();
     tdoc->m_createView = new TestStubs::TextView(0);
+    m_garbage << tdoc->m_createView;
     View* v2 = triggerAnnotationsOnView(doc);
     Q_ASSERT(v1 != v2);
 
@@ -149,15 +157,15 @@ void AnnotationManagerTest::multipleFiles()
 {
     // setup
     CoveredFile* f1  = createCoveredFile(m_lineNr, m_nrofCalls, m_someUrl);
-    m_manager->addCoverageData(f1);
+    m_annotationManager->addCoverageData(f1);
     int line2 = 20, nrofCalls2 = 5;
     CoveredFile* f2 = createCoveredFile(line2, nrofCalls2, KUrl("bar.cpp"));
-    m_manager->addCoverageData(f2);
+    m_annotationManager->addCoverageData(f2);
 
     KDevDocument* doc1 = createKDevDocument(f1->url());
-    m_manager->watch(doc1);
+    m_annotationManager->watch(doc1);
     KDevDocument* doc2 = createKDevDocument(f2->url());
-    m_manager->watch(doc2);
+    m_annotationManager->watch(doc2);
 
     // exercise
     View* v1 = triggerAnnotationsOnView(doc1);
@@ -173,9 +181,9 @@ void AnnotationManagerTest::notCovered()
     // setup
     m_lineNr = 5;
     CoveredFile* f  = createCoveredFile(5, m_nrofCalls, m_someUrl);
-    m_manager->addCoverageData(f);
+    m_annotationManager->addCoverageData(f);
     KDevDocument* doc = createKDevDocument(f->url());
-    m_manager->watch(doc);
+    m_annotationManager->watch(doc);
 
     // exercise
     View* v = triggerAnnotationsOnView(doc);
@@ -190,10 +198,10 @@ void AnnotationManagerTest::multiCoverageOneFile()
 {
     CoveredFile* f1 = createCoveredFile(m_lineNr, m_nrofCalls, m_someUrl);
     CoveredFile* f2 = createCoveredFile(m_lineNr+1, m_nrofCalls+1, m_someUrl);
-    m_manager->addCoverageData(f1);
-    m_manager->addCoverageData(f2);
+    m_annotationManager->addCoverageData(f1);
+    m_annotationManager->addCoverageData(f2);
     KDevDocument* doc = createKDevDocument(m_someUrl);
-    m_manager->watch(doc);
+    m_annotationManager->watch(doc);
 
     View* v = triggerAnnotationsOnView(doc);
     assertAnnoCallCountEquals(m_lineNr, m_nrofCalls, v);
@@ -205,13 +213,13 @@ void AnnotationManagerTest::stopWatching()
 {
     // setup
     CoveredFile* f = createCoveredFile(m_lineNr, m_nrofCalls, m_someUrl);
-    m_manager->addCoverageData(f);
+    m_annotationManager->addCoverageData(f);
     KDevDocument* doc = createKDevDocument(f->url());
-    m_manager->watch(doc);
+    m_annotationManager->watch(doc);
 
     // exercise
     View* v = triggerAnnotationsOnView(doc);
-    m_manager->stopWatching(doc);
+    m_annotationManager->stopWatching(doc);
 
     // verify
     KTextEditor::AnnotationViewInterface *anno =
@@ -222,6 +230,54 @@ void AnnotationManagerTest::stopWatching()
     KVERIFY(! model);
 }
 
+void AnnotationManagerTest::annotationsOnExistingView()
+{
+    // create and activate a text view _before_ activating coverage
+    // annotations on the document
+    
+    // setup
+    CoveredFile* f = createCoveredFile(m_lineNr, m_nrofCalls, m_someUrl);
+    m_annotationManager->addCoverageData(f);
+    KDevDocument* doc = createKDevDocument(f->url());
+
+    // create a view
+    m_docCtrl->emitTextDocumentCreated(doc);
+    QTest::qWait(50);
+    KTextEditor::Document* tdoc = doc->textDocument();
+    KTextEditor::View* v = tdoc->createView(m_widget);
+    QTest::qWait(50);
+
+    // exercise
+    m_annotationManager->watch(doc);
+
+    // verify
+    KTextEditor::AnnotationViewInterface *anno =
+            qobject_cast<KTextEditor::AnnotationViewInterface*>(v);
+    Q_ASSERT_X(anno, "annotationsOnExistingView", "Err?");
+    KVERIFY(anno->isAnnotationBorderVisible());
+    AnnotationModel* model = anno->annotationModel();
+    KVERIFY(model);
+    
+    assertAnnoCallCountEquals(m_lineNr, m_nrofCalls, v);
+}
+
+void AnnotationManagerTest::addRemoveAndReAddAnnotations()
+{
+    // setup
+    CoveredFile* f = createCoveredFile(m_lineNr, m_nrofCalls, m_someUrl);
+    m_annotationManager->addCoverageData(f);
+    KDevDocument* doc = createKDevDocument(f->url());
+
+    // exercise
+    m_annotationManager->watch(doc);
+    View* v = triggerAnnotationsOnView(doc);
+    m_annotationManager->stopWatching(doc);
+    m_annotationManager->watch(doc);
+    
+    // verify
+    assertAnnoCallCountEquals(m_lineNr, m_nrofCalls, v);
+}
+
 ////////////////// setup helpers /////////////////////////////////////////////
 
 void AnnotationManagerTest::initManager(CoveredFile* f)
@@ -234,10 +290,13 @@ void AnnotationManagerTest::initManager(CoveredFile* f)
 KDevDocument* AnnotationManagerTest::createKDevDocument(const KUrl& u)
 {
     TestStubs::KDevDocument* kdoc = new TestStubs::KDevDocument(m_core);
+    m_garbageDocs << kdoc;
     kdoc->m_url = u;
     TestStubs::TextDocument* doc = new TestStubs::TextDocument;
+    m_garbage << doc;
     kdoc->m_textDocument = doc;
     doc->m_createView = new TestStubs::TextView(0);
+    m_garbage << doc->m_createView;
     return kdoc;
 }
 
@@ -246,6 +305,7 @@ CoveredFile* AnnotationManagerTest::createCoveredFile(int line, int callCount, K
     CoveredFile* f = new CoveredFile();
     f->setUrl(url);
     f->setCallCount(line, callCount);
+    m_garbageFiles << f;
     return f;
 }
 
