@@ -1,4 +1,4 @@
-/* KDevelop xUnit plugin
+/* KDevelop coverage plugin
  *
  * Copyright 2008 Manuel Breugelmans <mbr.nxi@gmail.com>
  *
@@ -34,6 +34,7 @@ Q_OBJECT
 private slots:
     void init();
     void cleanup();
+    void cleanupTestCase();
 
     void singleCoveredFile();
     void multipleFiles();
