@@ -212,6 +212,7 @@ void SimpleRefactoring::startInteractiveRename(KDevelop::IndexedDeclaration decl
 
   //Since we don't yet know what the text should be replaced with, we just collect the top-contexts to process
   SimpleRefactoringCollector* collector = new SimpleRefactoringCollector(decl);
+  collector->setCollectConstructors(true);
 
   QDialog dialog;
 
@@ -290,13 +291,13 @@ void SimpleRefactoring::startInteractiveRename(KDevelop::IndexedDeclaration decl
   foreach(const KDevelop::IndexedTopDUContext &collected, collector->m_allUsingContexts) {
     QSet<int> hadIndices;
     foreach(const IndexedDeclaration &decl, collector->declarations()) {
-      uint usedDeclarationIndex = collected.data()->indexForUsedDeclaration(collector->declaration().data(), false);
+      uint usedDeclarationIndex = collected.data()->indexForUsedDeclaration(decl.data(), false);
       if(hadIndices.contains(usedDeclarationIndex))
         continue;
       hadIndices.insert(usedDeclarationIndex);
       DocumentChangeSet::ChangeResult result = applyChanges(originalName, replacementName, changes, collected.data(), usedDeclarationIndex);
       if(!result) {
-        KMessageBox::error(0, i18n("Applying changes failed: %1").arg(result.m_failureReason));
+        KMessageBox::error(0, i18n("Applying changes failed: %1", result.m_failureReason));
         return;
       }
     }
@@ -304,13 +305,14 @@ void SimpleRefactoring::startInteractiveRename(KDevelop::IndexedDeclaration decl
 
   DocumentChangeSet::ChangeResult result = applyChangesToDeclarations(originalName, replacementName, changes, collector->declarations());
   if(!result) {
-    KMessageBox::error(0, i18n("Applying changes failed: %1").arg(result.m_failureReason));
+    KMessageBox::error(0, i18n("Applying changes failed: %1", result.m_failureReason));
     return;
   }
 
-  result = changes.applyAllChanges();
+  ///We have to ignore failed changes for now, since uses of a constructor or of operator() may be created on "(" parens
+  result = changes.applyAllChanges(DocumentChangeSet::IgnoreFailedChange);
   if(!result) {
-      KMessageBox::error(0, i18n("Applying changes failed: %1").arg(result.m_failureReason));
+      KMessageBox::error(0, i18n("Applying changes failed: %1", result.m_failureReason));
       return;
   }
 }
