@@ -342,8 +342,9 @@ void DeclarationBuilder::visitDeclarator (DeclaratorAST* node)
                 continue; //Do not steal declarations
             }
 
-            if(FunctionDefinition* funDef = dynamic_cast<FunctionDefinition*>(currentDeclaration()))
+            if(FunctionDefinition* funDef = dynamic_cast<FunctionDefinition*>(currentDeclaration())) {
               funDef->setDeclaration(dec);
+            }
 
             found = true;
             break;
@@ -406,7 +407,7 @@ T* DeclarationBuilder::openDeclaration(NameAST* name, AST* rangeNode, const Iden
 }
 
 template<class T>
-T* DeclarationBuilder::openDeclarationReal(NameAST* name, AST* rangeNode, const Identifier& customName, bool collapseRangeAtStart, bool collapseRangeAtEnd)
+T* DeclarationBuilder::openDeclarationReal(NameAST* name, AST* rangeNode, const Identifier& customName, bool collapseRangeAtStart, bool collapseRangeAtEnd, const SimpleRange* customRange)
 {
   SimpleRange newRange;
   if(name) {
@@ -418,8 +419,10 @@ T* DeclarationBuilder::openDeclarationReal(NameAST* name, AST* rangeNode, const
       start = name->unqualified_name->tilde+1;
 
     newRange = editor()->findRange(start, end);
-  }else{
+  }else if(rangeNode) {
     newRange = editor()->findRange(rangeNode);
+  }else if(customRange) {
+    newRange = *customRange;
   }
 
   if(collapseRangeAtStart)
@@ -606,6 +609,12 @@ Cpp::ClassDeclaration* DeclarationBuilder::openClassDefinition(NameAST* name, AS
   DUChainWriteLocker lock(DUChain::lock());
   ret->setDeclarationIsDefinition(true);
   ret->clearBaseClasses();
+  
+  if(m_accessPolicyStack.isEmpty())
+    ret->setAccessPolicy(KDevelop::Declaration::Public);
+  else
+    ret->setAccessPolicy(currentAccessPolicy());
+  
   ret->setClassType(classType);
   return ret;
 }
@@ -883,6 +892,25 @@ void DeclarationBuilder::classContextOpened(ClassSpecifierAST *node, DUContext*
   currentDeclaration()->setInternalContext(context);
 }
 
+  void DeclarationBuilder::visitNamespace(NamespaceAST* ast) {
+
+  if (ast->namespace_name) {
+    DUChainWriteLocker lock(DUChain::lock());
+    SimpleRange range = editor()->findRange(ast->namespace_name, ast->namespace_name+1);
+    Identifier id(editor()->tokenToString(ast->namespace_name));
+    openDeclarationReal<Declaration>(0, 0, id, false, false, &range);
+  }
+  
+  DeclarationBuilderBase::visitNamespace(ast);
+  
+  if (ast->namespace_name) {
+    DUChainWriteLocker lock(DUChain::lock());
+    currentDeclaration()->setKind(KDevelop::Declaration::Namespace);
+    clearLastType();
+    closeDeclaration();
+  }
+}
+
 void DeclarationBuilder::visitClassSpecifier(ClassSpecifierAST *node)
 {
   PushValue<bool> setNotInTypedef(m_inTypedef, false);
@@ -1151,10 +1179,8 @@ void DeclarationBuilder::visitElaboratedTypeSpecifier(ElaboratedTypeSpecifierAST
     DeclarationBuilderBase::visitElaboratedTypeSpecifier(node);
     return;
   }
-
-  //For now completely ignore friend-class specifiers, because those currently are wrongly parsed as forward-declarations.
-  if( !m_storageSpecifiers.isEmpty() && (m_storageSpecifiers.top() & ClassMemberDeclaration::FriendSpecifier) )
-    return;
+  
+  bool isFriendDeclaration = !m_storageSpecifiers.isEmpty() && (m_storageSpecifiers.top() & ClassMemberDeclaration::FriendSpecifier);
 
   bool openedDeclaration = false;
 
@@ -1164,7 +1190,7 @@ void DeclarationBuilder::visitElaboratedTypeSpecifier(ElaboratedTypeSpecifierAST
 
     bool forwardDeclarationGlobal = false;
 
-    if(m_declarationHasInitDeclarators) {
+    if(m_declarationHasInitDeclarators || isFriendDeclaration) {
       /**This is an elaborated type-specifier
        *
        * See iso c++ draft 3.3.4 for details.
@@ -1193,6 +1219,11 @@ void DeclarationBuilder::visitElaboratedTypeSpecifier(ElaboratedTypeSpecifierAST
             //This belongs into the type-builder, but it's much easier to do here, since we already have all the information
             ///@todo See above, only search for fitting declarations(of structure/enum/class/union type)
             injectType(declarations.first()->abstractType());
+            
+            if( isFriendDeclaration ) {
+              lock.unlock();
+              createFriendDeclaration(node);
+            }
             return;
           }else{
             kDebug(9007) << "Error: Bad declaration";
@@ -1210,7 +1241,6 @@ void DeclarationBuilder::visitElaboratedTypeSpecifier(ElaboratedTypeSpecifierAST
 
         if(forwardDeclarationGlobal) {
           //Open the global context, so it is currentContext() and we can insert the forward-declaration there
-
           DUContext* globalCtx;
           {
             DUChainReadLocker lock(DUChain::lock());
@@ -1248,8 +1278,16 @@ void DeclarationBuilder::visitElaboratedTypeSpecifier(ElaboratedTypeSpecifierAST
       static_cast<ForwardDeclaration*>(currentDeclaration())->setResolved(idType->declaration());*/
     closeDeclaration();
   }
+  
+  if( isFriendDeclaration )
+    createFriendDeclaration(node);
 }
 
+void DeclarationBuilder::createFriendDeclaration(AST* range) {
+  static IndexedIdentifier friendIdentifier(Identifier("friend"));
+  openDeclaration<Declaration>(0, range, friendIdentifier.identifier(), true);
+  closeDeclaration();
+}
 
 void DeclarationBuilder::visitAccessSpecifier(AccessSpecifierAST* node)
 {
