@@ -61,6 +61,7 @@
 #include <unistd.h>
 #include <qwaitcondition.h>
 #include <language/duchain/duchainutils.h>
+#include "includepathcomputer.h"
 
 //#define DUMP_SMART_RANGES
 //#define DUMP_AST
@@ -68,6 +69,33 @@
 
 using namespace KDevelop;
 
+///Facilities to prevent multiple parse-jobs from processing the same url.
+QMutex urlParseMutex;
+QMap<IndexedString, QPair<Qt::HANDLE, uint> > parsingUrls;
+
+UrlParseLock::UrlParseLock(IndexedString url) : m_url(url) {
+  QMutexLocker lock(&urlParseMutex);
+  while(parsingUrls.contains(m_url) && parsingUrls[m_url].first != QThread::currentThreadId()) {
+    //Wait here until no other thread is updating parsing the url
+    lock.unlock();
+    sleep(1);
+    lock.relock();
+  }
+  if(parsingUrls.contains(m_url))
+    ++parsingUrls[m_url].second;
+  else
+    parsingUrls.insert(m_url, qMakePair(QThread::currentThreadId(), 1u));
+}
+
+UrlParseLock::~UrlParseLock() {
+  QMutexLocker lock(&urlParseMutex);
+  Q_ASSERT(parsingUrls.contains(m_url));
+  Q_ASSERT(parsingUrls[m_url].first == QThread::currentThreadId());
+  --parsingUrls[m_url].second;
+  if(parsingUrls[m_url].second == 0)
+    parsingUrls.remove(m_url);
+}
+
 bool importsContext(const QVector<DUContext::Import>& contexts, const DUContext* context) {
   foreach(const DUContext::Import &listCtx, contexts)
     if(listCtx.context(0) && listCtx.context(0)->imports(context))
@@ -106,13 +134,13 @@ void CPPParseJob::setNeedUpdateEverything(bool need) {
 }
 
 CPPParseJob::CPPParseJob( const KUrl &url,
-                    CppLanguageSupport *parent, PreprocessJob* parentPreprocessor )
-        : KDevelop::ParseJob( url, parent ),
+                    PreprocessJob* parentPreprocessor )
+        : KDevelop::ParseJob( url, 0 ),
         m_needUpdateEverything( false ),
         m_parentPreprocessor( parentPreprocessor ),
         m_session( new ParseSession ),
         m_readFromDisk( false ),
-        m_includePathsComputed( false ),
+        m_includePathsComputed( 0 ),
         m_keepDuchain( false ),
         m_parsedIncludes( 0 ),
         m_needsUpdate( true )
@@ -187,10 +215,8 @@ PreprocessJob* CPPParseJob::parentPreprocessor() const {
     return m_parentPreprocessor;
 }
 
-void CPPParseJob::gotIncludePaths(const KUrl::List& paths) {
-  m_includePathsComputed = true;
-  m_includePathUrls = paths;
-  m_includePaths = convertFromUrls(m_includePathUrls);
+void CPPParseJob::gotIncludePaths(IncludePathComputer* comp) {
+  m_includePathsComputed = comp;
   m_waitForIncludePathsMutex.lock(); //This makes sure that the parse thread goes into the waiting state first
   m_waitForIncludePathsMutex.unlock();
   m_waitForIncludePaths.wakeAll();
@@ -214,6 +240,11 @@ const QList<IndexedString>& CPPParseJob::includePaths() const {
             //Will be woken once the include-paths are computed
             m_waitForIncludePaths.wait(&m_waitForIncludePathsMutex);
             m_waitForIncludePathsMutex.unlock();
+            Q_ASSERT(m_includePathsComputed);
+            m_includePathsComputed->computeBackground();
+            m_includePathUrls = m_includePathsComputed->result();
+            m_includePaths = convertFromUrls(m_includePathUrls);
+            
         }
         return m_includePaths;
     } else {
@@ -223,6 +254,7 @@ const QList<IndexedString>& CPPParseJob::includePaths() const {
 
 CPPParseJob::~CPPParseJob()
 {
+  delete m_includePathsComputed;
   delete m_session;
 }
 
@@ -258,11 +290,7 @@ ReferencedTopDUContext CPPParseJob::updatingContentContext() const {
 
 CppLanguageSupport * CPPParseJob::cpp() const
 {
-    Q_ASSERT( parent() || parentPreprocessor() );
-    if( parent() )
-        return static_cast<CppLanguageSupport*>(const_cast<QObject*>(parent()));
-    else
-        return static_cast<CPPParseJob*>(parentPreprocessor()->parent())->cpp();
+  return CppLanguageSupport::self();
 }
 
 void CPPParseJob::addIncludedFile(KDevelop::ReferencedTopDUContext duChain, int sourceLine) {
@@ -316,6 +344,7 @@ const KTextEditor::Range& CPPParseJob::textRangeToParse() const
 CPPInternalParseJob::CPPInternalParseJob(CPPParseJob * parent)
     : ThreadWeaver::Job(parent)
     , m_priority(0)
+    , m_initialized(false)
 {
 }
 
@@ -337,9 +366,36 @@ LineContextPair contentFromProxy(LineContextPair ctx) {
     }
 }
 
+void CPPInternalParseJob::initialize() {
+    if(m_initialized)
+      return;
+    m_initialized = true;
+    
+    updatingProxyContext = parentJob()->updatingProxyContext().data();
+    updatingContentContext = parentJob()->updatingContentContext().data();
+
+    proxyContext = updatingProxyContext;
+    contentContext = updatingContentContext;
+/*    if(!contentContext)
+      contentContext = ContextBuilder::createEmptyTopContext(parentJob()->document());
+   
+    if(!proxyContext && Cpp::EnvironmentManager::isSimplifiedMatching())
+      proxyContext = ContextBuilder::createEmptyTopContext(parentJob()->document());*/
+}
+
 void CPPInternalParseJob::run()
 {
+    if(!CppLanguageSupport::self())
+      return;
+    //If we have a parent, the parse-mutex is already locked
+    QReadLocker lock(parentJob()->parentPreprocessor() ? 0 : parentJob()->cpp()->language()->parseLock());
+    if(!CppLanguageSupport::self())
+      return;
+    
+    UrlParseLock urlLock(parentJob()->document());
+    
     if(!parentJob()->needsUpdate()) {
+      parentJob()->processDelayedImports();
       kDebug( 9007 ) << "===-- ALREADY UP TO DATE --===> " << parentJob()->document().str();
       return;
     }
@@ -357,14 +413,7 @@ void CPPInternalParseJob::run()
 
     parentJob()->setLocalProgress(0, i18n("Parsing actual file"));
 
-    //If we have a parent, the parse-mutex is already locked
-    QReadLocker lock(parentJob()->parentPreprocessor() ? 0 : parentJob()->cpp()->language()->parseLock());
-    
-    ReferencedTopDUContext updatingProxyContext = parentJob()->updatingProxyContext().data();
-    ReferencedTopDUContext updatingContentContext = parentJob()->updatingContentContext().data();
-
-    ReferencedTopDUContext proxyContext = updatingProxyContext; //Proxy-context if simplified environment-matching is enabled, else zero.
-    ReferencedTopDUContext contentContext= updatingContentContext;  //The actual context that contains the data.
+    initialize();
 
     Cpp::EnvironmentFilePointer proxyEnvironmentFile(parentJob()->proxyEnvironmentFile());
     Cpp::EnvironmentFilePointer contentEnvironmentFile(parentJob()->contentEnvironmentFile());
@@ -414,13 +463,14 @@ void CPPInternalParseJob::run()
               //As above, we work with the content-contexts.
               LineContextPair context = contentFromProxy(topContext);
               DUContextPointer ptr(context.context);
-//               if( !importsContext(importedContentChains, context.context)  && (!updatingContentContext ||       !importsContext(updatingContentContext->importedParentContexts(), context.context)) ) {
-//                   if(!updatingContentContext || !updatingContentContext->imports(context.context, updatingContentContext->range().end)) {
+              if( !importsContext(importedContentChains, context.context)  && (!updatingContentContext ||       !importsContext(updatingContentContext->importedParentContexts(), context.context)) ) {
+                if(!updatingContentContext || !updatingContentContext->imports(context.context, updatingContentContext->range().end)) {
+                    //This must be conditional, else we might remove needed includes later because we think they were purely temporary
                     importedContentChains << context;
                     importedContentChains.back().temporary = true;
                     importedTemporaryChains << context.context;
-//                   }
-//               }
+                }
+              }
           }
           if(currentJob->parentPreprocessor())
             currentJob = currentJob->parentPreprocessor()->parentJob();
@@ -485,35 +535,10 @@ void CPPInternalParseJob::run()
 
       kDebug( 9007 ) << (contentContext ? "updating" : "building") << "duchain for" << parentJob()->document().str();
 
-      if(contentContext) {
-
-        ///We're updating, wait until no other thread is updating this context, and then mark that we are updating it.
-        bool wait = true;
-
-        while(wait) {
-          DUChainWriteLocker l(DUChain::lock());
-          if( contentContext->flags() & TopDUContext::UpdatingContext )
-            wait = true;
-          else
-            wait = false;
-
-          if(!wait)
-            contentContext->setFlags( (TopDUContext::Flags)(contentContext->flags() | TopDUContext::UpdatingContext) );
-
-          if(wait) {
-            kDebug(9007) << "waiting while" << parentJob()->document().str() << "is being updated by another thread";
-            l.unlock();
-            sleep(1);
-          }
-        }
-
-        DUChainWriteLocker l(DUChain::lock());
-        contentContext->clearProblems();
-      }
-      
       uint oldItemCount = 0;
       if(contentContext) {
-        DUChainReadLocker l(DUChain::lock());
+        DUChainWriteLocker l(DUChain::lock());
+        contentContext->clearProblems();
         oldItemCount = contentContext->childContexts().size() + contentContext->localDeclarations().size();
       }
 
@@ -523,6 +548,9 @@ void CPPInternalParseJob::run()
       if(contentContext)
         newFeatures = (TopDUContext::Features)(newFeatures | contentContext->features());
       
+      if(newFeatures & TopDUContext::ForceUpdate)
+        kDebug() << "update enforced";
+      
       ///At some point, we have to give up on features again, else processing will be just too slow.
       ///Simple solution for now: Always go down to the minimum required level.
       ///@todo Think of a good mechanism to decide when we want to drop features, and when we want to keep them.
@@ -605,7 +633,6 @@ void CPPInternalParseJob::run()
         contentContext->updateImportsCache();
       }
       
-
       if (!parentJob()->abortRequested()) {
         if (newFeatures & TopDUContext::AllDeclarationsContextsAndUses) {
             parentJob()->setLocalProgress(0.5, i18n("Building uses"));
@@ -613,11 +640,10 @@ void CPPInternalParseJob::run()
             UseBuilder useBuilder(&editor);
             useBuilder.buildUses(ast);
         }
-
+        
         if (!parentJob()->abortRequested() && editor.smart()) {
-          editor.smart()->clearRevision();
 
-          if ( parentJob()->cpp()->codeHighlighting() )
+          if ( parentJob()->cpp() && parentJob()->cpp()->codeHighlighting() )
           {
               parentJob()->cpp()->codeHighlighting()->highlightDUChain( contentContext );
           }
@@ -645,6 +671,17 @@ void CPPInternalParseJob::run()
       contentContext->updateImportsCache();
     }
 
+    {
+      DUChainReadLocker lock(DUChain::lock());
+      foreach(LineContextPair import, parentJob()->includedFiles()) {
+        if(import.temporary)
+          continue;
+        LineContextPair context = contentFromProxy(import);
+        Q_ASSERT(context.context);
+        Q_ASSERT(contentContext->imports(context.context.data(), SimpleCursor::invalid()));
+      }
+    }
+
     ///Build/update the proxy-context
 
     if( proxyEnvironmentFile ) {
@@ -667,20 +704,6 @@ void CPPInternalParseJob::run()
         foreach ( const LineContextPair &context, parentJob()->includedFiles() )
           proxyContext->addImportedParentContext(context.context, SimpleCursor(context.sourceLine, 0));
 
-        //Make sure we don't imported any not imported contexts
-        foreach(const DUContext::Import &import, proxyContext->importedParentContexts()) {
-          TopDUContext* top = dynamic_cast<TopDUContext*>(import.context(0));
-          Q_ASSERT(top);
-          bool shouldBeIncluded = false;
-          if(top == contentContext)
-            shouldBeIncluded = true;
-          foreach(const LineContextPair &ctx, parentJob()->includedFiles())
-            if(ctx.context == contentContext)
-              shouldBeIncluded = true;
-          if(!shouldBeIncluded)
-            proxyContext->removeImportedParentContext(top);
-        }
-          
         proxyContext->updateImportsCache();
           
         proxyContext->clearProblems();
@@ -737,11 +760,12 @@ void CPPInternalParseJob::run()
         //kDebug(9007) << "Dot-graph:\n" << dumpGraph.dotGraph(topContext, true);
     }
 
-
     //DumpTree dumpTree;
 
     kDebug( 9007 ) << "===-- Parsing finished --===>" << parentJob()->document().str();
 
+    parentJob()->processDelayedImports();
+    
     //Check the import structure
 /*    if(parentJob()->masterJob() == parentJob()) {
       DUChainReadLocker l(DUChain::lock());
@@ -758,6 +782,43 @@ void CPPInternalParseJob::run()
     }*/
 }
 
+void CPPParseJob::processDelayedImports() {
+  if(!m_delayedImports.isEmpty()) {
+    foreach(LineJobPair job, m_delayedImports)
+      job.first->addDelayedImporter(LineContextPair(m_parseJob->proxyContext ? m_parseJob->proxyContext : m_parseJob->contentContext, job.second));
+    m_delayedImports.clear();
+  }
+  if(!m_delayedImporters.isEmpty()) {
+    DUChainWriteLocker l(DUChain::lock());
+    foreach(LineContextPair context, m_delayedImporters) {
+      Q_ASSERT(context.context->parsingEnvironmentFile());
+      if(context.context->parsingEnvironmentFile()->isProxyContext()) {
+        Q_ASSERT(m_parseJob->proxyContext);
+        context.context->addImportedParentContext(m_parseJob->proxyContext.data(), SimpleCursor(context.sourceLine, 0));
+        Cpp::EnvironmentFile* cppEnvFile = dynamic_cast<Cpp::EnvironmentFile*>(context.context->parsingEnvironmentFile().data());
+        Q_ASSERT(cppEnvFile);
+        cppEnvFile->merge(dynamic_cast<Cpp::EnvironmentFile&>(*m_parseJob->proxyContext->parsingEnvironmentFile().data()));
+        context.context->updateImportsCache();
+      }
+      Q_ASSERT(m_parseJob->contentContext);
+      LineContextPair content = contentFromProxy(context);
+      Q_ASSERT(content.context);
+      content.context->addImportedParentContext(m_parseJob->proxyContext.data(), SimpleCursor(content.sourceLine, 0));
+      content.context->updateImportsCache();
+      Cpp::EnvironmentFile* cppEnvFile = dynamic_cast<Cpp::EnvironmentFile*>(content.context->parsingEnvironmentFile().data());
+      Q_ASSERT(cppEnvFile);
+      cppEnvFile->merge(dynamic_cast<Cpp::EnvironmentFile&>(*m_parseJob->contentContext->parsingEnvironmentFile().data()));
+    }
+  }
+}
+
+void CPPParseJob::addDelayedImport(LineJobPair job) {
+  m_delayedImports << job;
+}
+
+void CPPParseJob::addDelayedImporter(LineContextPair duChain) {
+  m_delayedImporters << duChain;
+}
 
 void CPPParseJob::requestDependancies()
 {
