@@ -46,6 +46,7 @@
 #include "cpppreprocessenvironment.h"
 #include "cppduchain/classdeclaration.h"
 #include <qstandarditemmodel.h>
+#include <language/duchain/functiondefinition.h>
 
 using namespace KTextEditor;
 
@@ -63,6 +64,13 @@ QString testFile3 = "struct A {}; struct B : public A {};";
 
 QString testFile4 = "void test1() {}; class TestClass() { TestClass() {} };";
 
+QStandardItemModel& fakeModel() {
+  static QStandardItemModel model;
+  model.setColumnCount(10);
+  model.setRowCount(10);
+  return model;
+}
+
 namespace QTest {
   template<>
   char* toString(const Cursor& cursor)
@@ -119,6 +127,36 @@ namespace QTest {
   }
 }
 
+//Helper-class for testing completion-items
+//Just initialize it with the context and the text, and then use the members, for simple cases only "names"
+struct CompletionItemTester {
+  CompletionItemTester(DUContext* context, QString text = "; ") {
+    completionContext = new  Cpp::CodeCompletionContext(DUContextPointer(context), text, QString());
+    bool abort = false;
+    items = completionContext->completionItems(context->range().end, abort);
+    foreach(Item i, items)
+      names << i->data(fakeModel().index(0, KTextEditor::CodeCompletionModel::Name), Qt::DisplayRole, 0).toString();
+  }
+  
+  QStringList names; //Names of all completion-items, not sorted
+  typedef KSharedPtr <KDevelop::CompletionTreeItem > Item;
+  QList <Item > items; //All items retrieved, sorted by name
+  
+  Cpp::CodeCompletionContext::Ptr completionContext;
+  
+  //Convenience-function to retrieve data from completion-items by name
+  QVariant itemData(QString itemName, int column = KTextEditor::CodeCompletionModel::Name, int role = Qt::DisplayRole) {
+    return itemData(names.indexOf(itemName), column, role);
+  }
+  
+  QVariant itemData(int itemNumber, int column = KTextEditor::CodeCompletionModel::Name, int role = Qt::DisplayRole) {
+    if(itemNumber < 0 || itemNumber >= items.size())
+      return QVariant();
+    
+    return items[itemNumber]->data(fakeModel().index(0, column), role, 0);
+  }
+};
+
 #define TEST_FILE_PARSE_ONLY if (testFileParseOnly) QSKIP("Skip", SkipSingle);
 TestCppCodeCompletion::TestCppCodeCompletion()
 {
@@ -177,15 +215,12 @@ void TestCppCodeCompletion::testPrivateVariableCompletion() {
   typedef KSharedPtr <KDevelop::CompletionTreeItem > Item;
   
   QList <Item > items = cptr->completionItems(context->range().end, abort);
-  QStandardItemModel fakeModel;
-  fakeModel.setColumnCount(10);
-  fakeModel.setRowCount(10);
   
   bool hadThis = false;
   
   foreach(Item i, items) {
     NormalDeclarationCompletionItem* decItem  = dynamic_cast<NormalDeclarationCompletionItem*>(i.data());
-    kDebug() << i->data(fakeModel.index(0, KTextEditor::CodeCompletionModel::Name), Qt::DisplayRole, 0).toString();
+    kDebug() << i->data(fakeModel().index(0, KTextEditor::CodeCompletionModel::Name), Qt::DisplayRole, 0).toString();
     if(decItem) {
       kDebug() << typeid(*i.data()).name();
       QVERIFY(decItem);
@@ -193,7 +228,7 @@ void TestCppCodeCompletion::testPrivateVariableCompletion() {
     }else{
       TypeConversionCompletionItem* conversion = dynamic_cast<TypeConversionCompletionItem*>(i.data());
       QVERIFY(conversion);
-      QCOMPARE(conversion->data(fakeModel.index(0, KTextEditor::CodeCompletionModel::Name), Qt::DisplayRole, 0).toString(), QString("this"));
+      QCOMPARE(conversion->data(fakeModel().index(0, KTextEditor::CodeCompletionModel::Name), Qt::DisplayRole, 0).toString(), QString("this"));
       hadThis = true;
     }
   }
@@ -204,8 +239,68 @@ void TestCppCodeCompletion::testPrivateVariableCompletion() {
   release(context);
 }
 
-void TestCppCodeCompletion::testSameNamespace() {
+void TestCppCodeCompletion::testInheritanceVisibility() {
   TEST_FILE_PARSE_ONLY
+  QByteArray method("class A { public: class AMyClass {}; }; class B : protected A { public: class BMyClass {}; }; class C : private B{ public: class CMyClass {}; }; class D : public C { class DMyClass{}; }; ");
+  TopDUContext* top = parse(method, DumpNone);
+
+  DUChainWriteLocker lock(DUChain::lock());
+
+  QCOMPARE(top->childContexts().count(), 4);
+
+  QCOMPARE(top->childContexts()[1]->type(), DUContext::Class);
+  QVERIFY(top->childContexts()[1]->owner());
+  QVERIFY(Cpp::localClassFromCodeContext(top->childContexts()[1]));
+  //From within B, MyClass is visible, because of the protected inheritance
+  QCOMPARE(CompletionItemTester(top->childContexts()[1], "A::").names, QStringList() << "AMyClass");
+  QCOMPARE(CompletionItemTester(top->childContexts()[1]).names.toSet(), QSet<QString>() << "BMyClass" << "AMyClass" << "A" << "B" );
+  QCOMPARE(CompletionItemTester(top, "A::").names, QStringList() << "AMyClass");
+  QCOMPARE(CompletionItemTester(top, "B::").names, QStringList() << "BMyClass");
+  QCOMPARE(CompletionItemTester(top->childContexts()[2]).names.toSet(), QSet<QString>() << "CMyClass" << "BMyClass" << "AMyClass" << "C" << "B" << "A");
+  QCOMPARE(CompletionItemTester(top, "C::").names.toSet(), QSet<QString>() << "CMyClass");
+  QCOMPARE(CompletionItemTester(top->childContexts()[3]).names.toSet(), QSet<QString>() << "DMyClass" << "CMyClass" << "D" << "C" << "B" << "A");
+  QCOMPARE(CompletionItemTester(top, "D::").names.toSet(), QSet<QString>() << "CMyClass" ); //DMyClass is private
+}
+
+void TestCppCodeCompletion::testFriendVisibility() {
+  TEST_FILE_PARSE_ONLY
+  QByteArray method("class A { class PrivateClass {}; friend class B; }; class B{};");
+  TopDUContext* top = parse(method, DumpNone);
+
+  DUChainWriteLocker lock(DUChain::lock());
+
+  QCOMPARE(top->childContexts().count(), 2);
+
+  //No type within A, so there should be no items
+  QCOMPARE(CompletionItemTester(top->childContexts()[1], "A::").names, QStringList() << "PrivateClass");
+}
+
+void TestCppCodeCompletion::testNamespaceCompletion() {
+  
+  QByteArray method("namespace A { class m; namespace Q {}; }; namespace A { class n; int q; }");
+  TopDUContext* top = parse(method, DumpAll);
+
+  DUChainWriteLocker lock(DUChain::lock());
+
+  QCOMPARE(top->localDeclarations().count(), 2);
+  QCOMPARE(top->childContexts().count(), 2);
+  QCOMPARE(top->localDeclarations()[0]->identifier(), Identifier("A"));
+  QCOMPARE(top->localDeclarations()[1]->identifier(), Identifier("A"));
+  QCOMPARE(top->localDeclarations()[0]->kind(), Declaration::Namespace);
+  QCOMPARE(top->localDeclarations()[1]->kind(), Declaration::Namespace);
+  QVERIFY(!top->localDeclarations()[0]->abstractType());
+  QVERIFY(!top->localDeclarations()[1]->abstractType());
+  QCOMPARE(top->localDeclarations()[0]->internalContext(), top->childContexts()[0]);
+  QCOMPARE(top->localDeclarations()[1]->internalContext(), top->childContexts()[1]);
+  
+  QCOMPARE(CompletionItemTester(top).names, QStringList() << "A");
+
+  QCOMPARE(CompletionItemTester(top->childContexts()[1], "A::").names.toSet(), QSet<QString>() << "m" << "n" << "Q");
+  QCOMPARE(CompletionItemTester(top).itemData("A", KTextEditor::CodeCompletionModel::Prefix).toString(), QString("namespace"));
+  release(top);
+}
+
+void TestCppCodeCompletion::testSameNamespace() {
 
   //                 0         1         2         3         4         5         6         7
   //                 0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012
@@ -218,24 +313,14 @@ void TestCppCodeCompletion::testSameNamespace() {
   QVERIFY(!top->parentContext());
   QCOMPARE(top->childContexts().count(), 2);
   QCOMPARE(top->childContexts()[1]->childContexts().count(), 2);
+  QCOMPARE(top->childContexts()[1]->localDeclarations().count(), 1);
+  FunctionDefinition* funDef = dynamic_cast<KDevelop::FunctionDefinition*>(top->childContexts()[1]->localDeclarations()[0]);
+  QVERIFY(!funDef->hasDeclaration());
 
 //   lock.unlock();
   {
-    Cpp::CodeCompletionContext::Ptr cptr( new  Cpp::CodeCompletionContext(DUContextPointer(top->childContexts()[1]->childContexts()[1]), "; ", QString()) );
-    bool abort = false;
-    typedef KSharedPtr <KDevelop::CompletionTreeItem > Item;
-    
-    QList <Item > items = cptr->completionItems(top->range().end, abort);
-    QStandardItemModel fakeModel;
-    foreach(Item i, items) {
-      NormalDeclarationCompletionItem* decItem  = dynamic_cast<NormalDeclarationCompletionItem*>(i.data());
-      QVERIFY(decItem);
-      kDebug() << decItem->declaration()->toString();
-      kDebug() << i->data(fakeModel.index(0, KTextEditor::CodeCompletionModel::Name), Qt::DisplayRole, 0).toString();
-    }
-    
-    //Have been filtered out, because only types are shown from the global scope
-    QCOMPARE(items.count(), 3); //C, test2, test
+    kDebug() << CompletionItemTester(top->childContexts()[1]->childContexts()[1]).names;
+    QCOMPARE(CompletionItemTester(top->childContexts()[1]->childContexts()[1]).names.toSet(), QSet<QString>() << "C" << "test2" << "test" << "A");
   }
   
   release(top);
@@ -264,12 +349,11 @@ void TestCppCodeCompletion::testUnnamedNamespace() {
     typedef KSharedPtr <KDevelop::CompletionTreeItem > Item;
     
     QList <Item > items = cptr->completionItems(top->range().end, abort);
-    QStandardItemModel fakeModel;
     foreach(Item i, items) {
       NormalDeclarationCompletionItem* decItem  = dynamic_cast<NormalDeclarationCompletionItem*>(i.data());
       QVERIFY(decItem);
       kDebug() << decItem->declaration()->toString();
-      kDebug() << i->data(fakeModel.index(0, KTextEditor::CodeCompletionModel::Name), Qt::DisplayRole, 0).toString();
+      kDebug() << i->data(fakeModel().index(0, KTextEditor::CodeCompletionModel::Name), Qt::DisplayRole, 0).toString();
     }
     
     //Have been filtered out, because only types are shown from the global scope
@@ -281,12 +365,11 @@ void TestCppCodeCompletion::testUnnamedNamespace() {
     typedef KSharedPtr <KDevelop::CompletionTreeItem > Item;
     
     QList <Item > items = cptr->completionItems(top->range().end, abort);
-    QStandardItemModel fakeModel;
     foreach(Item i, items) {
       NormalDeclarationCompletionItem* decItem  = dynamic_cast<NormalDeclarationCompletionItem*>(i.data());
       QVERIFY(decItem);
       kDebug() << decItem->declaration()->toString();
-      kDebug() << i->data(fakeModel.index(0, KTextEditor::CodeCompletionModel::Name), Qt::DisplayRole, 0).toString();
+      kDebug() << i->data(fakeModel().index(0, KTextEditor::CodeCompletionModel::Name), Qt::DisplayRole, 0).toString();
     }
     
     QCOMPARE(items.count(), 3); //b, a, and test
@@ -914,11 +997,9 @@ void TestCppCodeCompletion::testMacroExpansionRanges() {
   QString test("#define TEST namespace NS{int a;int b;int c;int d;int q;} class A{}; \nTEST; int a; int b; int c; int d;int e;int f;int g;int h;\n");
   DUChainWriteLocker l(DUChain::lock());
   TopDUContext* ctx = parse(test.toUtf8());
-  QCOMPARE(ctx->localDeclarations().count(), 9);
-  kDebug() << ctx->localDeclarations()[0]->range().textRange();
-  kDebug() << ctx->localDeclarations()[1]->range().textRange();
-  QCOMPARE(ctx->localDeclarations()[0]->range().textRange(), KTextEditor::Range(1, 4, 1, 4)); //Because the macro TEST was expanded out of its physical range, the Declaration is collapsed.
-  QCOMPARE(ctx->localDeclarations()[1]->range().textRange(), KTextEditor::Range(1, 10, 1, 11));
+  QCOMPARE(ctx->localDeclarations().count(), 10);
+  QCOMPARE(ctx->localDeclarations()[1]->range().textRange(), KTextEditor::Range(1, 4, 1, 4)); //Because the macro TEST was expanded out of its physical range, the Declaration is collapsed.
+  QCOMPARE(ctx->localDeclarations()[2]->range().textRange(), KTextEditor::Range(1, 10, 1, 11));
 }
 {
   //The range of the merged declaration name should be trimmed to the end of the macro invocation
