@@ -340,7 +340,8 @@ void CMakeProjectVisitor::defineTarget(const QString& id, const QStringList& sou
 
 int CMakeProjectVisitor::visit(const AddExecutableAst *exec)
 {
-    defineTarget(exec->executable(), exec->sourceLists(), Executable);
+    if(!exec->isImported())
+        defineTarget(exec->executable(), exec->sourceLists(), Executable);
     kDebug(9042) << "exec:" << exec->executable() << "->" << m_filesPerTarget[exec->executable()]
         << "was" << exec->content()[exec->line()].writeBack();
     return 1;
@@ -497,11 +498,11 @@ int CMakeProjectVisitor::visit(const FindPackageAst *pack)
         QString var="CMAKE_INSTALL_PREFIX";
         QString instPath;
         if(m_vars->contains(var))
-            instPath = m_vars->value(var)[0];
+            instPath = m_vars->value(var).join(QString());
         else if(m_cache->contains(var))
             instPath = m_cache->value(var);
 
-        kDebug(9042) << "config mode" << m_vars->value(var)[0] << m_cache->value(var) << instPath;
+//         kDebug(9042) << "config mode" << m_vars->value(var).join(QString()) << m_cache->value(var) << instPath;
 
 #if defined(Q_OS_WIN)
         modulePath.prepend(instPath);
@@ -1873,10 +1874,10 @@ int CMakeProjectVisitor::walk(const CMakeFileContent & fc, int line)
             element = new MacroCallAst;
 
         createUses(*it);
-//         kDebug(9042) << "resolving:" << it->writeBack();
+        kDebug(9042) << "resolving:" << it->writeBack();
         CMakeFunctionDesc func = resolveVariables(*it); //FIXME not correct in while case
         bool correct = element->parseFunctionInfo(func);
-//         kDebug(9042) << "resolved:" << func.writeBack() << correct;
+        kDebug(9042) << "resolved:" << func.writeBack() << correct;
         if(!correct)
         {
             kDebug(9042) << "error! found an error while processing" << func.writeBack() << "was" << it->writeBack() << endl
@@ -1944,8 +1945,9 @@ void CMakeProjectVisitor::createUses(const CMakeFunctionDesc& desc)
         if(!arg.isCorrect() || !arg.value.contains('$'))
             continue;
 
-        QList< IntPair > var = parseArgument(arg.value);
-        for(QList<IntPair>::const_iterator it=var.constBegin(); it!=var.constEnd(); ++it)
+        QList<IntPair> var = parseArgument(arg.value);
+        QList<IntPair>::const_iterator it, itEnd=var.constEnd();
+        for(it=var.constBegin(); it!=itEnd; ++it)
         {
             QString var=arg.value.mid(it->first+1, it->second-it->first-1);
             QList<Declaration*> decls=m_topctx->findDeclarations(Identifier(var));
